@@ -9,7 +9,7 @@ use components_arena::{Arena, Component, NewtypeComponentId, Id};
 use dep_obj::{dep_type, dep_obj};
 use macro_attr_2018::macro_attr;
 use dep_obj::binding::{Bindings, Binding1, Binding2};
-use dyn_context::{State, StateExt};
+use dyn_context::state::{State, StateExt};
 use std::any::{TypeId, Any};
 use std::borrow::Cow;
 use std::fmt::Write;
@@ -40,9 +40,19 @@ impl Item {
         game.items.insert(|id| (ItemData { props: ItemProps::new_priv() }, Item(id)))
     }
 
+    fn drop_item(self, state: &mut dyn State) {
+        self.drop_bindings_priv(state);
+        let game: &mut Game = state.get_mut();
+        game.items.remove(self.0);
+    }
+
     dep_obj! {
-        fn props(self as this, game: Game) -> &mut ItemProps {
-            &mut game.items[this.0].props
+        fn props(self as this, game: Game) -> ItemProps {
+            if mut {
+                &mut game.items[this.0].props
+            } else {
+                &game.items[this.0].props
+            }
         }
     }
 }
@@ -70,27 +80,38 @@ impl Npc {
     fn new(state: &mut dyn State) -> Npc {
         let game: &mut Game = state.get_mut();
         let npc = game.npcs.insert(|id| (NpcComponent { props: NpcProps::new_priv() }, Npc(id)));
-        let removed_items_binding = Binding1::new(&mut game.bindings, |x| Some(x));
+        let removed_items_binding = Binding1::new(state, |x| Some(x));
         removed_items_binding.set_source_1(state, &mut NpcProps::EQUIPPED_ITEMS.removed_items_source(npc.props()));
-        removed_items_binding.handle_fn(state, (), |state, (), items| {
+        removed_items_binding.set_target_fn(state, (), |state, (), items| {
             for item in items {
                 ItemProps::EQUIPPED.set_distinct(state, item.props(), false);
             }
         });
-        let game: &mut Game = state.get_mut();
-        let inserted_items_binding = Binding1::new(&mut game.bindings, |x| Some(x));
+        npc.props().add_binding(state, removed_items_binding.into());
+        let inserted_items_binding = Binding1::new(state, |x| Some(x));
         inserted_items_binding.set_source_1(state, &mut NpcProps::EQUIPPED_ITEMS.inserted_items_source(npc.props()));
-        inserted_items_binding.handle_fn(state, (), |state, (), items| {
+        inserted_items_binding.set_target_fn(state, (), |state, (), items| {
             for item in items {
                 ItemProps::EQUIPPED.set_distinct(state, item.props(), true);
             }
         });
+        npc.props().add_binding(state, inserted_items_binding.into());
         npc
     }
 
+    fn drop_npc(self, state: &mut dyn State) {
+        self.drop_bindings_priv(state);
+        let game: &mut Game = state.get_mut();
+        game.npcs.remove(self.0);
+    }
+
     dep_obj! {
-        fn props(self as this, game: Game) -> &mut NpcProps {
-            &mut game.npcs[this.0].props
+        fn props(self as this, game: Game) -> NpcProps {
+            if mut {
+                &mut game.npcs[this.0].props
+            } else {
+                &game.npcs[this.0].props
+            }
         }
     }
 }
@@ -144,13 +165,14 @@ fn main() {
     let shield = Item::new(game);
     ItemProps::NAME.set_uncond(game, shield.props(), Cow::Borrowed("Shield"));
     for item in [sword, shield] {
-        let log = Binding2::new(&mut game.bindings, |(old, new), (_, name)| if old == new { None} else { Some((new, name)) });
+        let log = Binding2::new(game, |(old, new), (_, name)| if old == new { None } else { Some((new, name)) });
         log.set_source_1(game, &mut ItemProps::EQUIPPED.source(item.props()));
         log.set_source_2(game, &mut ItemProps::NAME.source(item.props()));
-        log.handle_fn(game, (), |game, (), (equipped, name)| {
+        log.set_target_fn(game, (), |game, (), (equipped, name)| {
             let game: &mut Game = game.get_mut();
             writeln!(&mut game.log, "{} {}.", name, if equipped { "equipped" } else { "unequipped" }).unwrap();
         });
+        item.props().add_binding(game, log.into());
     }
     NpcProps::EQUIPPED_ITEMS.push(game, npc.props(), sword);
     NpcProps::EQUIPPED_ITEMS.push(game, npc.props(), shield);
@@ -160,4 +182,7 @@ fn main() {
         Shield equipped.\n\
         Sword unequipped.\n\
     ");
+    npc.drop_npc(game);
+    sword.drop_item(game);
+    shield.drop_item(game);
 }
